using System;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace Lexer
{

    public class LexerException : System.Exception
    {
        public LexerException(string msg)
            : base(msg)
        {
        }

    }

    public class Lexer
    {

        protected int position;
        protected char currentCh; //   
        protected int currentCharValue; //     
        protected System.IO.StringReader inputReader;
        protected string inputString;

        public Lexer(string input)
        {
            inputReader = new System.IO.StringReader(input);
            inputString = input;
        }

        public void Error()
        {
            System.Text.StringBuilder o = new System.Text.StringBuilder();
            o.Append(inputString + '\n');
            o.Append(new System.String(' ', position - 1) + "^\n");
            o.AppendFormat("Error in symbol {0}", currentCh);
            throw new LexerException(o.ToString());
        }

        protected void NextCh()
        {
            this.currentCharValue = this.inputReader.Read();
            this.currentCh = (char) currentCharValue;
            this.position += 1;
        }

        public virtual bool Parse()
        {
            return true;
        }
    }

    public class IntLexer : Lexer
    {

        protected System.Text.StringBuilder intString;
        public int parseResult = 0;

        public IntLexer(string input)
            : base(input)
        {
            intString = new System.Text.StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            int sign = 1;
            if (currentCh == '+' || currentCh == '-')
            {
				if(currentCh== '-')
				{
					sign = -1;
				}
                NextCh();
            }
        
            if (char.IsDigit(currentCh))
            {
				parseResult = parseResult * 10 + (currentCh - 48);
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
				parseResult = parseResult * 10 + (currentCh - 48);
				NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            parseResult*=sign;

            return true;

        }
    }
    
    public class IdentLexer : Lexer
    {
        public string parseResult;
        protected StringBuilder builder;
    
        public string ParseResult
        {
            get { return parseResult; }
        }
    
        public IdentLexer(string input) : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if (currentCh >= 'A' && currentCh <= 'Z')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            if (currentCh >= 'a' && currentCh <= 'z')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || currentCh >= 'A' && currentCh <= 'Z' || currentCh >= 'a' && currentCh <= 'z' || currentCh == '_')
            {
                parseResult += currentCh;
                NextCh();
            }

            if (currentCharValue != -1)
            {
                Error();
            }

            return true;
            // throw new NotImplementedException();
        }
       
    }

    public class IntNoZeroLexer : IntLexer
    {
        protected System.Text.StringBuilder intString;
        public int parseResult = 0;

        public IntNoZeroLexer(string input)
            : base(input)
        {
            intString = new System.Text.StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            int sign = 1;
            if (currentCh == '+' || currentCh == '-')
            {
                if (currentCh == '-')
                {
                    sign = -1;
                }
                NextCh();
            }

            if (char.IsDigit(currentCh) && currentCh != '0')
            {
                parseResult = parseResult * 10 + (currentCh - 48);
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
                parseResult = parseResult * 10 + (currentCh - 48);
                NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            parseResult *= sign;

            return true;
            //throw new NotImplementedException();
        }
    }

    public class LetterDigitLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public LetterDigitLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            bool is_dig = true;

            if (currentCh >= 'A' && currentCh <= 'Z' || currentCh >= 'a' && currentCh <= 'z')
            {
                is_dig = false;
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || currentCh >= 'A' && currentCh <= 'Z' || currentCh >= 'a' && currentCh <= 'z')
            {
                if(char.IsDigit(currentCh) && is_dig || (currentCh >= 'A' && currentCh <= 'Z' || currentCh >= 'a' && currentCh <= 'z') && !is_dig)
                {
                    Error();
                }
                else
                    if(char.IsDigit(currentCh))
                {
                    is_dig = true;
                    parseResult += currentCh;
                    NextCh();
                }
                else
                    if(currentCh >= 'A' && currentCh <= 'Z' || currentCh >= 'a' && currentCh <= 'z')
                {
                    is_dig = false;
                    parseResult += currentCh;
                    NextCh();
                }
            }

            if (currentCharValue != -1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
       
    }

    public class LetterListLexer : Lexer
    {
        protected List<char> parseResult;

        public List<char> ParseResult
        {
            get { return parseResult; }
        }

        public LetterListLexer(string input)
            : base(input)
        {
            parseResult = new List<char>();
        }

        public override bool Parse()
        {
            NextCh();

            bool is_letter = false;
            int count_letters = 0;
            int count_spacers = 0;

            if (char.IsLetter(currentCh))
            {
                is_letter = true;
                count_letters++;
                parseResult.Add(currentCh);
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsLetter(currentCh) || currentCh == ',' || currentCh == ';')
            {
                if (char.IsLetter(currentCh) && is_letter || (currentCh == ',' || currentCh == ';') && !is_letter)
                {
                    Error();
                }
                else
                    if (char.IsLetter(currentCh))
                {
                    is_letter = true;
                    count_letters++;
                    parseResult.Add(currentCh);
                    NextCh();
                }
                else
                if (currentCh == ',' || currentCh == ';')
                {
                    is_letter = false;
                    count_spacers++;
                    NextCh();
                }
            }

            if (currentCharValue != -1 || count_letters != count_spacers + 1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
    }

    public class DigitListLexer : Lexer
    {
        protected List<int> parseResult;

        public List<int> ParseResult
        {
            get { return parseResult; }
        }

        public DigitListLexer(string input)
            : base(input)
        {
            parseResult = new List<int>();
        }

        public override bool Parse()
        {
            NextCh();

            bool is_dig = false;
            int k = 0;//  
            int c = 1;//  

            while (char.IsDigit(currentCh) || currentCh == ' '){

                if (char.IsDigit(currentCh) && !is_dig)
                {
                    is_dig = true;
                    k++;
                    string s = "";
                    for (int i = 0; i < 1; i++)
                    {
                        s += currentCh;
                    }
                    parseResult.Add(Int32.Parse(s));
                    NextCh();
                    c++;
                    
                }
                else
                    if (currentCh == ' ' && k!=0)
                {
                    is_dig = false;
                    NextCh();
                    c++;
                }
                else
                    break;
            
            }

            if (k == 1 && c > 2 || currentCharValue != -1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
    }

    public class LetterDigitGroupLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }
        
        public LetterDigitGroupLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if(char.IsDigit(currentCh) || currentCharValue ==-1)
            {
                Error();
            }

            int seq_digit_count = 0; //   
            int seq_letter_count = 0; //   
            bool is_dig = false;

            while (char.IsDigit(currentCh) || char.IsLetter(currentCh))
            {
                if (char.IsDigit(currentCh) && seq_digit_count < 2)
                {
                    is_dig = true;
                    seq_digit_count++;
                    seq_letter_count = 0;
                    parseResult += currentCh;
                    NextCh();
                }
                else
                    if (char.IsLetter(currentCh) && seq_letter_count < 2)
                {
                    is_dig = false;
                    seq_letter_count++;
                    seq_digit_count = 0;
                    parseResult += currentCh;
                    NextCh();
                }
                else
                    break;
            }
                if (currentCharValue != -1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
       
    }

    public class DoubleLexer : Lexer
    {
        private StringBuilder builder;
        private double parseResult;

        public double ParseResult
        {
            get { return parseResult; }

        }

        public DoubleLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            if (currentCh == '.' || currentCharValue == -1)
            {
                Error();
            }

            bool all_integer_part = false;
            double denum = 10;

            while(char.IsDigit(currentCh) || currentCh == '.')
            {
                if (char.IsDigit(currentCh) && !all_integer_part)
                {
                    string s = "";
                    s += currentCh;
                    parseResult = parseResult * 10 + Int32.Parse(s);
                    NextCh();
                }
                else
                if (currentCh == '.')
                {
                    all_integer_part = true;
                    NextCh();
                }
                else
                    if (char.IsDigit(currentCh) && all_integer_part)
                {
                    string s = "";
                    s += currentCh;
                    parseResult = parseResult + (Int32.Parse(s) / denum);
                    denum *= 10;
                    NextCh();
                }
                else
                    break;
            }

            if (all_integer_part && denum == 10 || currentCharValue != -1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
       
    }

    public class StringLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public StringLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            bool open_lexem = false;
            bool close_lexem = false;
            int count = 0; // 

            if(currentCh!='\'')
            {
                Error();
            }

            while (!close_lexem && currentCharValue!=-1)
            {
                if (currentCh == '\'' && !open_lexem)
                {
                    count++;
                    open_lexem = true;
                }
                else
                if (currentCh == '\'' && open_lexem)
                {
                    count++;
                    close_lexem = true;
                }
                parseResult += currentCh;
                NextCh();
            }

            if (count != 2 || currentCharValue!=-1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
    }

    public class CommentLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public CommentLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {

            NextCh();

            bool open_comment = false;
            bool close_comment = false;
            int count = 0; //  '/'

            if (currentCh != '/')
            {
                Error();
            }
            NextCh();
            if (currentCh != '*')
            {
                Error();
            }
            NextCh();
            count++;

            while (!close_comment && currentCharValue != -1)
            {
                if (currentCh == '*')
                {
                    NextCh();
                    if(currentCharValue!=-1 && currentCh == '/')
                    {
                        count++;
                        close_comment = true;
                    }
                }
                parseResult += currentCh;
                NextCh();
            }

            if (count != 2 || currentCharValue != -1)
            {
                Error();
            }

            return true;
            //throw new NotImplementedException();
        }
    }

    public class IdentChainLexer : Lexer
    {
        private StringBuilder builder;
        private List<string> parseResult;

        public List<string> ParseResult
        {
            get { return parseResult; }

        }

        public IdentChainLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
            parseResult = new List<string>();
        }

        public override bool Parse()
        {
            throw new NotImplementedException();
        }
    }

    public class Program
    {
        public static void Main()
        {
            string input = "''";
            Lexer L = new StringLexer(input);
            try
            {
                L.Parse();
            }
            catch (LexerException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}