%{
//      GPPGParser,   ,   gppg
    public BlockNode root; //     
    public Parser(AbstractScanner<ValueType, LexLocation> scanner) : base(scanner) { }
%}

%output = SimpleYacc.cs

%union { 
			public double dVal; 
			public int iVal; 
			public string sVal; 
			public Node nVal;
			public ExprNode eVal;
			public StatementNode stVal;
			public BlockNode blVal;
			public OpNode opVal;
       }

%using ProgramTree;

%namespace SimpleParser

%token BEGIN END CYCLE ASSIGN SEMICOLON WHILE DO REPEAT UNTIL FOR TO WRITE LEFT_BRACKET RIGHT_BRACKET IF THEN ELSE VAR COMMA PLUS MINUS MULT DIVISION
%token <iVal> INUM 
%token <dVal> RNUM 
%token <sVal> ID

%type <eVal> expr ident T F
%type <stVal> assign statement cycle while repeat for write if idlist
%type <blVal> stlist block
%type <opVal> plus minus mult div

%%

progr   : block { root = $1; }
		;

stlist	: statement 
			{ 
				$$ = new BlockNode($1); 
			}
		| stlist SEMICOLON statement 
			{ 
				$1.Add($3); 
				$$ = $1; 
			}
		;

idlist	: VAR ident
			{
				$$ = new VarDefNode($2 as IdNode); 
			}
		| idlist COMMA ident
			{
				($1 as VarDefNode).Add($3 as IdNode);
				$$ = $1;
			}
		;

statement: assign { $$ = $1; }
		| block   { $$ = $1; }
		| cycle   { $$ = $1; }
		| while	  { $$ = $1; }
		| repeat  { $$ = $1; }
		| for     { $$ = $1; }
		| write   { $$ = $1; }
		| if	  { $$ = $1; }
		| idlist  { $$ = $1; }
	;

ident 	: ID { $$ = new IdNode($1); }	
		;
	
assign 	: ident ASSIGN expr { $$ = new AssignNode($1 as IdNode, $3); }
		;

expr	: ident  { $$ = $1 as IdNode; }
		| INUM { $$ = new IntNumNode($1); }
		;

block	: BEGIN stlist END { $$ = $2; }
		;

cycle	: CYCLE expr statement { $$ = new CycleNode($2, $3); }
		;

while	: WHILE expr DO statement { $$ = new WhileNode($2, $4); }
		;

repeat  : REPEAT stlist UNTIL expr { $$ = new RepeatNode($2, $4); }
		;

for		: FOR ID ASSIGN expr TO expr DO statement { $$ = new ForNode($4,$6,$8); }
		;

write   : WRITE LEFT_BRACKET expr RIGHT_BRACKET { $$ = new WriteNode($3); }
		;

if		: IF expr THEN statement { $$ = new IfNode($2,$4); }
		| IF expr THEN statement ELSE statement { $$ = new IfFullNode($2,$4,$6); }
		;

plus	: PLUS { $$ = new OpNode($1.ToString()); }
		;

minus	: MINUS { $$ = new OpNode($1.ToString()); }
		;

mult	: MULT { $$ = new OpNode($1.ToString()); }
		;

div		: DIVISION { $$ = new OpNode($1.ToString()); }
		;

expr	: T
			{
				$$ = $1;
			}
		| expr plus T
			{
				$$ = new BinaryNode($1,$3,$2);
			}
		| expr minus T
			{
				$$ = new BinaryNode($1,$3,$2);
			}
		;

T		:  F
			{
				$$ = $1;
			}
		|
		  T mult F
			{
				$$ = new BinaryNode($1,$3,$2);
			}
		|
		  T div F
			{
				$$ = new BinaryNode($1,$3,$2);
			}
		;

F		: ident
			{
				$$ = $1 as IdNode;
			}
		|
		  INUM
			{
				$$ = new IntNumNode($1);
			}
		| LEFT_BRACKET expr RIGHT_BRACKET
			{
				$$ = $2;
			}
		;

%%

